package com.muyu.data.source.service.impl;

import java.sql.*;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import com.muyu.common.core.domain.Result;
import com.muyu.common.core.utils.ObjUtils;
import com.muyu.common.security.utils.SecurityUtils;
import com.muyu.data.source.domain.*;
import com.muyu.data.source.domain.model.DatabaseTableModel;
import com.muyu.data.source.domain.req.DataSourceSaveReq;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.muyu.data.source.mapper.DataSourceMapper;
import com.muyu.data.source.service.DataSourceService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;

/**
 * 数据源Service业务层处理
 *
 * @author muyu
 * @date 2024-04-21
 */
@Slf4j
@Service
public class DataSourceServiceImpl extends ServiceImpl<DataSourceMapper, DataSource>  implements DataSourceService {
    @Autowired
    private DataSourceMapper dataSourceMapper;

    /**
     * 查询数据源列表
     *
     * @param dataSource 数据源
     * @return 数据源
     */
    @Override
    public List<DataSource> list(DataSource dataSource) {
        LambdaQueryWrapper<DataSource> queryWrapper = new LambdaQueryWrapper<>();


        if (ObjUtils.notNull(dataSource.getAccessSourceName())){
            queryWrapper.like(DataSource::getAccessSourceName, dataSource.getAccessSourceName());
        }

        if (ObjUtils.notNull(dataSource.getDataSourceSystemName())){
            queryWrapper.like(DataSource::getDataSourceSystemName, dataSource.getDataSourceSystemName());
        }

        if (ObjUtils.notNull(dataSource.getHostAddress())){
            queryWrapper.eq(DataSource::getHostAddress, dataSource.getHostAddress());
        }

        if (ObjUtils.notNull(dataSource.getHostPort())){
            queryWrapper.eq(DataSource::getHostPort, dataSource.getHostPort());
        }

        if (ObjUtils.notNull(dataSource.getDataAccessTypeId())){
            queryWrapper.eq(DataSource::getDataAccessTypeId, dataSource.getDataAccessTypeId());
        }

        if (ObjUtils.notNull(dataSource.getDatabaseName())){
            queryWrapper.like(DataSource::getDatabaseName, dataSource.getDatabaseName());
        }

        if (ObjUtils.notNull(dataSource.getDataConnectionParameter())){
            queryWrapper.eq(DataSource::getDataConnectionParameter, dataSource.getDataConnectionParameter());
        }

        if (ObjUtils.notNull(dataSource.getInitialQuantity())){
            queryWrapper.eq(DataSource::getInitialQuantity, dataSource.getInitialQuantity());
        }

        if (ObjUtils.notNull(dataSource.getMaximumQuantity())){
            queryWrapper.eq(DataSource::getMaximumQuantity, dataSource.getMaximumQuantity());
        }



        if (ObjUtils.notNull(dataSource.getMaximumFrequency())){
            queryWrapper.eq(DataSource::getMaximumFrequency, dataSource.getMaximumFrequency());
        }





        return list(queryWrapper);
    }

    @Override
    public List<AccessType> findAccessType() {
        return dataSourceMapper.findAccessType();
    }

    @Override
    public Result testConnection(DataSourceSaveReq dataSourceQueryReq) {
        String user = dataSourceQueryReq.getDatabaseUserName();
        String password = dataSourceQueryReq.getDatabaseUserPassword();
        String jdbcDriver = "com.mysql.cj.jdbc.Driver";
        String url = "jdbc:mysql://"+dataSourceQueryReq.getHostAddress()+":"+dataSourceQueryReq.getHostPort()+"/"+dataSourceQueryReq.getDatabaseName();
        try {
            Connection con = DriverManager.getConnection(url, user, password);
            if (con!=null){
                return Result.success("连接成功");
            }
            Statement stmt = con.createStatement();

        } catch (SQLException e) {
           log.error("连接失败："+e.getMessage());
            return Result.error("连接失败");
        }

      return Result.success();
    }

    @Override
    public Result synchronization(DataSource dataSource) {

        AssetStructure assetStructure = new AssetStructure();

        ArrayList<String> tableNames = new ArrayList<>();
        List<DatabaseTableInformation> databaseTableInformationList=dataSourceMapper.findByBaseName(dataSource.getDatabaseName());
        dataSourceMapper.addDatabaseTableInformation(databaseTableInformationList);
        List<AssetStructure> list = new ArrayList<>();

                String user=dataSource.getDatabaseUserName();
                String password=dataSource.getDatabaseUserPassword();
                String jdbcDriver="com.mysql.cj.jdbc.Driver";
                String url = "jdbc:mysql://" + dataSource.getHostAddress() + ":" + dataSource.getHostPort() + "/" + dataSource.getDatabaseName();
                String query="show tables  ";
                try(Connection con=DriverManager.getConnection(url,user,password);
                    Statement stmt=con.createStatement();
                    ResultSet rs= stmt.executeQuery(query)) {
                    ResultSetMetaData rsmd = rs.getMetaData();
                    // 5. 处理查询结果
                    while (rs.next()) {
                        String tableName = rs.getString(1);
                        tableNames.add(tableName);
                    }

                        assetStructure.setAccessSourceName(dataSource.getAccessSourceName());
                        assetStructure.setDatabaseName(dataSource.getDatabaseName());
                        assetStructure.setDataSourceSystemName(dataSource.getDataSourceSystemName());

                            assetStructure.setDatabaseTableName(rsmd.getTableName(1));

                            assetStructure.setDataVolume(rsmd.getColumnDisplaySize(1));
                            assetStructure.setDataQuantity(rsmd.getColumnDisplaySize(1));

                     dataSourceMapper.addAssets1(assetStructure);

                    return Result.success("同步成功");

                } catch (SQLException e) {
                    System.out.println("连接失败"+e.getMessage());
                    return Result.error("同步失败");
                }

    }

    @Override
    public Result<List<DatabaseTableModel>> findDataBaseTable() {
        List<AssetStructure> assetStructureList=dataSourceMapper.assetStructureList();
        List<DatabaseTableModel> databaseTableList = new ArrayList<>();
        for (AssetStructure assetStructure : assetStructureList) {
            DatabaseTableModel databaseTableModel=dataSourceMapper.findByName(assetStructure.getDatabaseName(),assetStructure.getDatabaseTableName());
            System.out.println(databaseTableModel);
            databaseTableList.add(databaseTableModel);
        }
        return Result.success(databaseTableList);
    }


    @Override
    public Result<List<AssetStructure>> findAssetStructure() {
        List<AssetStructure> assetStructureList=dataSourceMapper.assetStructureList();
        return Result.success(assetStructureList);
    }

    @Override
    public Result<Structure> findStructure() {
        Structure structure = new Structure();
        List<AssetStructure> assetStructure = dataSourceMapper.findAssetStructure();
        structure.setAssetStructureList(assetStructure);
        List<DatabaseTableInformation> databaseTableInformationList=dataSourceMapper.findInformation();
        structure.setDatabaseTableInformationList(databaseTableInformationList);
        return Result.success(structure);
    }


}
